/*
 *  $Id: dmpiatom.h,v 1.17 1994/04/15 18:55:57 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

/* Atomic types */

#ifndef _DMPIATOM_INCLUDE
#define _DMPIATOM_INCLUDE

/*****************************************************************************
*                           MPI ATOMIC DATA STRUCTURES                       *
*****************************************************************************/

/* insure that this is at least 32 bits */
typedef unsigned long uint32;

typedef struct _MPIR_HBT_node {
  void *value;
  int   keyval;
  short balance;
  struct _MPIR_HBT_node *left;
  struct _MPIR_HBT_node *right;
} MPIR_HBT_node;

typedef struct _MPIR_HBT {
  unsigned int height;
  MPIR_HBT_node *root;
} MPIR_HBT;

typedef struct  {
  int  (*copy_fn)();
  int  (*delete_fn)();
  void  *extra_state;
  int    ref_count;
  int    permanent;             /* Used to mark the permanent attributes of
				   MPI_COMM_WORLD */
} MPIR_Attr_key;

typedef struct {
  int nnodes;
  int nedges;
  int *index;
  int *edges;
} MPIR_GRAPH_TOPOLOGY;

typedef struct {
  int nnodes;
  int ndims;
  int *dims;
  int *periods;
  int *position;
} MPIR_CART_TOPOLOGY;

typedef struct {
  int type;
  union {
    MPIR_GRAPH_TOPOLOGY  graph;
    MPIR_CART_TOPOLOGY   cart;
  } tag;
} MPIR_TOPOLOGY;

typedef struct {
    int np;			        /* Number of processes in group */
    int local_rank;         /* My rank in the group (if I belong) */
    int ref_count;          /* Number of references to this group */
    int N2_next;            /* Next power of 2 from np */
    int N2_prev;            /* Previous power of 2 from np */
    int *lrank_to_grank;    /* Mapping from local to "global" ranks */
    int *set_mark;          /* Used for set marking/manipulation on groups */
} MPIR_GROUP;

typedef unsigned long MPIR_CONTEXT;
#define  MPIR_CONTEXT_TYPE MPI_UNSIGNED_LONG

#define  MPIR_WORLD_PT2PT_CONTEXT 0
#define  MPIR_WORLD_COLL_CONTEXT  1
#define  MPIR_SELF_PT2PT_CONTEXT  2
#define  MPIR_SELF_COLL_CONTEXT   3
#define  MPIR_FIRST_FREE_CONTEXT  4

typedef enum { MPIR_INTRA=0, MPIR_INTER } MPIR_COMM_TYPE;


#ifdef __STDC__
/* 
typedef void (MPI_Handler_function)(struct _MPIR_COMMUNICATOR *, int *, ... ); 
 */
typedef void (MPI_Handler_function)(struct _MPIR_COMMUNICATOR *, int *, 
				    char *, char *, int  );
#else
typedef void (MPI_Handler_function)();
#endif

/* We don't do anything special with the handler function */
typedef MPI_Handler_function *MPI_Errhandler;


typedef struct _MPIR_COMMUNICATOR {
    MPIR_COMM_TYPE comm_type;	/* inter or intra */
    MPIR_GROUP    *group;	/* group associated with communicator */
    MPIR_GROUP    *local_group;  /* local group */
    struct _MPIR_COMMUNICATOR *comm_coll; /* communicator for collective ops */

    int            ref_count;  /* number of references to communicator */
    MPIR_CONTEXT   send_context;  /* context to send messages */
    MPIR_CONTEXT   recv_context;  /* context to recv messages */
    void          *comm_cache;	/* Hook for communicator cache */
    MPIR_HBT      *attr_cache;   /* Hook for attribute cache */
    MPIR_TOPOLOGY  topology;     /* Topology information */
    MPI_Errhandler error_handler;  /* User routine to call */
} MPIR_COMMUNICATOR;

typedef enum {
    NO = 0, YES
} MPIR_BOOL;

typedef enum {
    MPIR_INT, MPIR_FLOAT, MPIR_DOUBLE, MPIR_COMPLEX, MPIR_LONG, MPIR_SHORT,
    MPIR_CHAR, MPIR_BYTE, MPIR_UCHAR, MPIR_USHORT, MPIR_ULONG, MPIR_UINT,
    MPIR_CONTIG, MPIR_VECTOR, MPIR_HVECTOR, MPIR_INDEXED,
    MPIR_HINDEXED, MPIR_STRUCT, MPIR_DOUBLE_COMPLEX, MPIR_PACKED, 
	MPIR_UB, MPIR_LB, MPIR_LONGDOUBLE 
} MPIR_NODETYPE;

#include "mpi_bc.h"
#endif
