/*
 *  $Id: mpi_bc.h,v 1.15 1994/04/13 13:39:48 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

/* user include file for MPI programs, with no dependencies */

#ifndef _MPI_INCLUDE_BC
#define _MPI_INCLUDE_BC

/* assorted defined constants */
#include "mpi_errno.h"              /* Error codes */

#define MPI_UNDEFINED      (-32766)
#define MPI_UNDEFINED_RANK MPI_UNDEFINED
#define MPI_KEYVAL_INVALID 0
#define MPI_NULL_FN ((void (*))0)

/* Topology types */
#define MPI_GRAPH  0
#define MPI_CART   1

/* Generic error handling code.  This handles inserting the file and line
   number (in MPI) where the error occured.  In addition, it
   checks the error handler and calls the appropriate one.  Finally, 
   it returns the errorcode as its value */
#define MPIR_ERROR(comm,code,string) \
    MPIR_Error( comm, code, string, __FILE__, __LINE__ )
    
/* communication modes */
typedef enum { 
    MPIR_MODE_STANDARD = 0, 
    MPIR_MODE_READY, 
    MPIR_MODE_SYNCHRONOUS, 
    MPIR_MODE_RECV,
    MPIR_MODE_SYNC_ACK
	
} MPIR_Mode;

#define MPI_BOTTOM      0

#define MPI_PROC_NULL  (-2)

/* Value of tag in status for a cancelled message */
#define MPIR_MSG_CANCELLED (-3)

typedef void *MPI_Buffer;

typedef struct { 
    int count;
    int MPI_SOURCE;
    int MPI_TAG;
    int count_in_bytes;     /* This is needed to implement MPI_Get_count
			       and MPI_Get_elements, particularly after
			       a probe */
} MPI_Status;

/* Must be able to hold any valid address.  64 bit machines may need
   to change this */
typedef long MPI_Aint;

/* This is the only global state in MPI */
extern int MPIR_Has_been_initialized;
extern int MPIR_Buffer_size;
extern void *MPIR_Buffer;

#endif

