/*
 *  $Id: chinit.c,v 1.11 1994/04/18 20:42:59 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char SCCSid[] = "%W% %G%";
#endif

/* 
    This file contains the routines that provide the basic information 
    on the device, and initialize it
 */

#include "mpid.h"
#include "dmch.h"

MPID_INFO *MPID_procinfo = 0;
MPID_H_TYPE MPID_byte_order;
void (*MPID_ErrorHandler)() = MPID_DefaultErrorHandler;

static int DebugSpace = 0;

MPID_SetSpaceDebugFlag( flag )
int flag;
{
DebugSpace = flag;
if (flag) {
    /* Check the validity of the malloc arena on every use of trmalloc/free */
    trDebugLevel( 1 );
    }
}

void MPID_CH_myrank( rank )
int *rank;
{
*rank = PImytid;
}

void MPID_CH_mysize( size )
int *size;
{
*size = PInumtids;
}

/* 
    In addition, Chameleon processes many command-line arguments 
 */
void MPID_CH_init( argc, argv )
int  *argc;
char **argv;
{
int  i;
char *work;

PIiInit( argc, argv );
/* Turn off the resource monitors */
#if !defined(euih)
/* If we are euih, we can't use SIGALRM; this call sets SIGALRM to 
   SIG_IGN */
SYSetResourceClockOff();
#endif

#ifdef MPID_HAS_HETERO
MPID_procinfo = (MPID_INFO *)MALLOC( PInumtids * sizeof(MPID_INFO) );
CHKPTR(MPID_procinfo);
for (i=0; i<PInumtids; i++) {
    MPID_procinfo[i].byte_order = MPID_H_NONE;
    }
/* Set my byte ordering and convert if necessary.  Eventually, this
   should use xdr */
i = SYGetByteOrder( );
if (i == 1)      MPID_byte_order = MPID_H_LSB;
else if (i == 2) MPID_byte_order = MPID_H_MSB;
else             MPID_byte_order = MPID_H_XDR;
MPID_procinfo[PImytid].byte_order = MPID_byte_order;
/* Everyone uses the same format (MSB) */
if (i == 1) 
    SYByteSwapInt( &MPID_procinfo[PImytid].byte_order, 1 );

/* Get everyone else's */
work = (char *)MALLOC( PInumtids * sizeof(MPID_INFO) );
CHKPTR( work );
/* ASSUMES MPID_INFO is ints */
PIgior( MPID_procinfo, PInumtids, work, PSAllProcs );
FREE( work );

#endif
}

void MPID_CH_Abort( code )
int code;
{
fprintf( stderr, "Aborting program!\n" );
fflush( stderr );
fflush( stdout );
SYexitall( (char *)0, code );
}

void MPID_CH_End()
{
PIiFinish();
if (MPID_procinfo) 
    FREE( MPID_procinfo );
if (DebugSpace)
    trdump( stdout );

}

void MPID_CH_Node_name( name, len )
char *name;
int  len;
{
PINodeName( name, len );
}

extern double SYGetElapsedTime();
double MPID_CH_Wtime()
{
return SYGetElapsedTime();
}

/* This returns a value that is correct but not the best value that
   could be returned */
double MPID_CH_Wtick()
{
double t1, t2;
int    cnt;

t1 = MPID_CH_Wtime();
cnt = 1000;
while (cnt-- && (t2 = MPID_CH_Wtime()) <= t1) ;
if (!cnt) return 1.0e6;
return t2 - t1;
}

void MPID_CH_Error_handler( r )
void (*r)();
{
if (r)
    MPID_ErrorHandler = r;
else
    MPID_ErrorHandler = MPID_DefaultErrorHandler;
}

void MPID_DefaultErrorHandler( code, str )
int  code;
char *str;
{
if (str) 
    fprintf( stderr, "%s\n", str );
MPID_CH_Abort( code );
}
