/*
 *  $Id: scan.c,v 1.19 1994/04/13 21:57:08 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: scan.c,v 1.19 1994/04/13 21:57:08 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "mpisys.h"
#include "coll.h"

/*@

MPI_Scan - Compute the scan (partial reductions) of data on a collection of
           processes

Input Parameters:
. sendbuf - starting address of send buffer (choice) 
. count - number of elements in input buffer (integer) 
. datatype - data type of elements of input buffer (handle) 
. op - operation (handle) 
. comm - communicator (handle) 

Output Parameter:
. recvbuf - starting address of receive buffer (choice) 
@*/
int MPI_Scan ( sendbuf, recvbuf, count, datatype, op, comm )
void             *sendbuf;
void             *recvbuf;
int               count;
MPI_Datatype      datatype;
MPI_Op            op;
MPI_Comm          comm;
{
  MPI_Status status;
  int        rank, size;
  int        errno = MPI_SUCCESS;
  MPI_Aint   extent;
  MPI_Uop   *uop;
  int        flag; 

  /* Check for invalid arguments */
  if ( ( (comm            == MPI_COMM_NULL) && (errno = MPI_ERR_COMM) )   ||
       ( (op              == MPI_OP_NULL)   && (errno = MPI_ERR_OP) )     ||
       ( ((count>0)&&(sendbuf==(void *)0))  && (errno = MPI_ERR_BUFFER) ) ||
       ( ((count>0)&&(recvbuf==(void *)0))  && (errno = MPI_ERR_BUFFER) ) ||
	   ( (sendbuf         == recvbuf)       && (errno = MPI_ERR_BUFFER) ) )
    return MPIR_ERROR( comm, errno, "Error in MPI_SCAN" );
  /* We also need to check that the datatype is a basic type? */

  /* Check for intra-communicator */
  MPI_Comm_test_inter ( comm, &flag );
  if (flag) 
    return MPIR_ERROR(comm, MPI_ERR_COMM,
					  "Inter-communicator invalid in MPI_SCAN");

  /* Get my rank & size and switch communicators to the hidden collective */
  MPI_Comm_size ( comm, &size );
  MPI_Comm_rank ( comm, &rank );
  MPI_Type_extent ( datatype, &extent );
  comm = comm->comm_coll;
  uop = op->op;

  /* Do the scan operation */
  if (rank > 0) {
	MPI_Recv(recvbuf,count,datatype,rank-1,MPIR_SCAN_TAG,comm,&status);
	(*uop)(sendbuf, recvbuf, &count, &datatype); 
  }
  else {
	MPI_Sendrecv(sendbuf,count,datatype,rank, MPIR_SCAN_TAG,
				 recvbuf,count,datatype,rank, MPIR_SCAN_TAG,
				 comm, &status);
  }
  
  /* send the letter to destination */
  if (rank < (size-1)) 
    MPI_Send(recvbuf,count,datatype,rank+1,MPIR_SCAN_TAG,comm);
  
  return(errno);
}
