/*
 *  $Id: dmpipk.c,v 1.2 1994/04/19 15:57:48 gropp Exp $
 *
 *  (C) 1994 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: dmpipk.c,v 1.2 1994/04/19 15:57:48 gropp Exp $";
#endif

#include "mpi.h"
#include "mpisys.h"

/* 
   This file contains the first pass at routines to pack and unpack datatypes
   for the ADI.  THESE WILL CHANGE
 */

/* Pack for a send.  Eventually, this will need to handle the Heterogeneous 
   case.  */
void MPIR_Pack_Hvector( buf, count, datatype, dest, outbuf )
char        *buf;
int         count, dest;
MPI_Datatype datatype;
char        *outbuf;
{
int count1 = datatype->count,           /* Number of blocks */
    blocklen = datatype->blocklen;      /* Number of elements in each block */
MPI_Aint    stride   = datatype->stride;  /* Bytes between blocks */
int extent = datatype->old_type->extent;  /* Extent of underlying type */
int blen   = blocklen * extent;
int c, i;

c = count * count1;
/* Handle the special case of 4 or 8 byte items, with appropriate 
   alignment.  We do this to avoid the cost of a memcpy call for each
   element.
 */
if (blen == 4 && ((int)buf & 0x3) == 0 && (stride & 0x3) == 0 && 
    sizeof(int) == 4) {
    register int *outb = (int *)outbuf, *inb = (int *)buf;
    stride = stride >> 2;
    for (i=0; i<c; i++) {
	outb[i] = *inb;
	inb    += stride;
	}
    }
else if (blen == 8 && ((int)buf & 0x7) == 0 && (stride & 0x7) == 0 && 
	 sizeof(double) == 8) {
    register double *outb = (double *)outbuf, *inb = (double *)buf;
    stride = stride >> 3;
    for (i=0; i<c; i++) {
	outb[i] = *inb;
	inb    += stride;
	}
    }
else {
    for (i=0; i<c; i++) {
	memcpy( outbuf, buf, blen );
	outbuf += blen; 
	buf    += stride;
	}
    }
}

void MPIR_UnPack_Hvector( inbuf, count, datatype, source, outbuf )
char        *inbuf;
int         count, source;
MPI_Datatype datatype;
char        *outbuf;
{
int count1 = datatype->count,            /* Number of blocks */
    blocklen = datatype->blocklen;       /* Number of elements in each block */
MPI_Aint    stride   = datatype->stride; /* Bytes between blocks */
int extent = datatype->old_type->extent;  /* Extent of underlying type */
int blen   = blocklen * extent;
register int c, i;

c = count * count1;
if (blen == 4 && ((int)inbuf & 0x3) == 0 && (stride & 0x3) == 0 && 
    sizeof(int) == 4) {
    register int *outb = (int *)outbuf, *inb = (int *)inbuf;
    stride = stride >> 2;
    for (i=0; i<c; i++) {
	*outb = inb[i];
	outb  += stride;
	}
    }
else if (blen == 8 && ((int)inbuf & 0x7) == 0 && (stride & 0x7) == 0 && 
	 sizeof(double) == 8) {
    register double *outb = (double *)outbuf, *inb = (double *)inbuf;
    stride = stride >> 3;
    for (i=0; i<c; i++) {
	*outb   = inb[i];
	outb    += stride;
	}
    }
else {
    for (i=0; i<c; i++) {
	memcpy( outbuf, inbuf, blen );
	outbuf += stride;
	inbuf  += blen;
	}
    }
}

/* Get the length needed for the Hvector as a contiguous lump */
int MPIR_HvectorLen( count, datatype )
int count;
MPI_Datatype datatype;
{
return datatype->size * count;
}

/*
    These routines pack/unpack data for messages.  They KNOW how the 
    message request areas are arranged, and how the send/receive routines
    known when to use these routines.
 */
int MPIR_PackMessage( buf, count, datatype, dest, request )
char *buf;
int  count, dest;
MPI_Datatype datatype;
MPI_Request request;
{
/*
if (!datatype->committed) 
    return MPI_ERR_TYPE;
 */
request->chandle.bufpos = (char *)MALLOC( datatype->size * count );
if (!request->chandle.bufpos) {
    return MPI_ERR_EXHAUSTED;
    }
if (datatype->dte_type == MPIR_HVECTOR) {
    MPIR_Pack_Hvector( buf, count, datatype, dest, request->chandle.bufpos );
    request->shandle.datatype = datatype->old_type;
    request->shandle.dev_shandle.start = request->chandle.bufpos;
    request->shandle.dev_shandle.bytes_as_contig = count * datatype->size;
    return MPI_SUCCESS;
    }
return MPI_ERR_INTERN;
}

int MPIR_EndPackMessage( request )
MPI_Request request;
{
FREE( request->chandle.bufpos );
return MPI_SUCCESS;
}

int MPIR_SetupUnPackMessage( buf, count, datatype, source, request )
char *buf;
int  count, source;
MPI_Datatype datatype;
MPI_Request request;
{
/*
if (!datatype->committed) 
    return MPI_ERR_TYPE;
 */
request->chandle.bufpos = (char *)MALLOC( datatype->size * count );
if (!request->chandle.bufpos) {
    return MPI_ERR_EXHAUSTED;
    }
request->rhandle.dev_rhandle.bytes_as_contig = datatype->size * count;
request->rhandle.dev_rhandle.start           = 
    request->chandle.bufpos;
return MPI_SUCCESS;
}

int MPIR_UnPackMessage( buf, count, datatype, source, request )
char *buf;
int  count, source;
MPI_Datatype datatype;
MPI_Request request;
{
if (datatype->dte_type == MPIR_HVECTOR) {
    MPIR_UnPack_Hvector( request->chandle.bufpos, count, datatype, 
			 source, buf );
    FREE( request->chandle.bufpos );
    return MPI_SUCCESS;
    }
return MPI_ERR_INTERN;
}
