/*
 *  $Id: testall.c,v 1.3 1994/04/12 16:01:33 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

/*@
    MPI_Testall - Test for the completion of all previously initiated
    communications

Input Parameters:
. count - lists length (integer) 
. array_of_requests - array of requests (array of handles) 
Output Parameters:
. flag - (logical) 
. array_of_statuses - array of status objects (array of Status) 
@*/
int MPI_Testall( count, array_of_requests, flag, array_of_statuses )
int        count;
MPI_Request array_of_requests[];
int        *flag;
MPI_Status *array_of_statuses;
{
    int i, found;
    MPI_Status status;

    MPID_check_device( 0 );
    found = 1;
    for (i = 0; i < count; i++)
	{
	if ( array_of_requests[i] != NULL ) {
	    
	    if ( array_of_requests[i]->chandle.completed == YES) {
		if ( array_of_requests[i]->type == MPIR_SEND ) {
		    MPID_complete_send( &array_of_requests[i]->shandle, 
				        &status );
		    }
		else {
		    MPID_complete_recv( &array_of_requests[i]->rhandle );
		    
		    array_of_statuses[i].MPI_SOURCE = 
			array_of_requests[i]->rhandle.source;
		    array_of_statuses[i].MPI_TAG    = 
			array_of_requests[i]->rhandle.tag;
		    array_of_statuses[i].count      = 
			array_of_requests[i]->rhandle.actcount;
		    array_of_statuses[i].count_in_bytes =
			array_of_requests[i]->rhandle.totallen;
		    }
		MPI_Request_free( &array_of_requests[i] );
		array_of_requests[i]    = NULL;
		}
	    else 
		found = 0;
	    }
	}
    *flag = found;	
    return MPI_SUCCESS;
}    
