/*
 *  $Id: testany.c,v 1.12 1994/04/12 16:01:41 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

/*@
    MPI_Testany - Test for completion of any previdously initiated 
                  communication

Input Parameters:
. count - list length (integer) 
. array_of_requests - array of requests (array of handles) 

Output Parameters:
. index - index of operation that completed, 
or MPI_UNDEFINED  if none completed (integer) 
. flag - true if one of the operations is complete (logical) 
. status - status object (Status) 
@*/
int MPI_Testany( count, array_of_requests, index, flag, status )
int         count;
MPI_Request array_of_requests[];
int         *index, *flag;
MPI_Status  *status;
{
    int i, found;

    MPID_check_device( 0 );
    found = 0;
    for (i = 0; i < count; i++)
    {
	if ( array_of_requests[i] != NULL )
  	if ( array_of_requests[i]->chandle.completed == YES) {
	    found         = 1;
	    *index        = i;
	    if ( array_of_requests[i]->type == MPIR_SEND ) {
		MPID_complete_send( &array_of_requests[i]->shandle, status );
	        }
	    else {
		MPID_complete_recv( &array_of_requests[i]->rhandle );

		status->MPI_SOURCE = array_of_requests[i]->rhandle.source;
		status->MPI_TAG	   = array_of_requests[i]->rhandle.tag;
		status->count	   = array_of_requests[i]->rhandle.actcount;
		status->count_in_bytes = 
			         array_of_requests[i]->rhandle.totallen;
	    }
 	    MPI_Request_free( &array_of_requests[i] );
 	    array_of_requests[i]    = NULL;
	    break;
        }
    }
    *flag = found;
    return MPI_SUCCESS;
}

