/*
 *  $Id: type_struct.c,v 1.1 1994/04/18 15:47:07 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

#ifndef MPIR_TRUE
#define MPIR_TRUE  1
#define MPIR_FALSE 0
#endif


/*@
    MPI_Type_struct - Create a struct datatype
@*/
int MPI_Type_struct( count, blocklens, indices, old_types, newtype )
int           count;
int 	      blocklens[];
MPI_Aint      indices[];      
MPI_Datatype  old_types[];
MPI_Datatype *newtype;
{
  MPIR_DATATYPE  *dteptr;
  MPI_Aint ub, lb, high, low;
  int      i, errno = MPI_SUCCESS;

  /* Check for bad arguments */
  if ( (count    <= 0) && (errno = MPI_ERR_COUNT) )
	return MPIR_ERROR( MPI_COMM_NULL, errno,
					  "Error in MPI_TYPE_STRUCT" );
	
  /* Create and fill in the datatype */
  dteptr = (*newtype) = (MPI_Datatype) SBalloc( MPIR_dtes );
  dteptr->dte_type    = MPIR_STRUCT;
  dteptr->committed   = MPIR_FALSE;
  dteptr->basic       = MPIR_FALSE;
  dteptr->is_contig   = MPIR_FALSE;
  dteptr->ref_count   = 1;
  dteptr->align       = old_types[0]->align;
  dteptr->count       = count;
  dteptr->elements    = 0;
  dteptr->size        = 0;

  /* Create indices and blocklens arrays and fill them */
  dteptr->indices     = ( int * ) MALLOC( count * sizeof( int ) );
  dteptr->blocklens   = ( int * ) MALLOC( count * sizeof( int ) );
  dteptr->old_types   =
	( MPIR_DATATYPE ** ) MALLOC( count * sizeof( MPIR_DATATYPE *));
  low                 = indices[0] * old_types[0]->extent;
  high                = low + (blocklens[0] * old_types[0]->extent);
  for (i = 0; i < count; i++)  {
	dteptr->old_types[i]  = (MPI_Datatype)MPIR_Type_dup (old_types[i]);
	dteptr->indices[i]    = indices[i];
	dteptr->blocklens[i]  = blocklens[i];
	ub = (indices[i]   * old_types[i]->extent) + 
	     (blocklens[i] * old_types[i]->extent) ;
	lb = (indices[i] * old_types[i]->extent);
	if ( high < ub )
	  high = ub;
	if ( low  > lb )
	  low  = lb;
	dteptr->elements     += (blocklens[i] * old_types[i]->elements);
	dteptr->size         += (blocklens[i] * old_types[i]->extent);   
  }

  /* Set the upper/lower bounds and the extent and size */
  dteptr->lb          = low;
  dteptr->ub          = high;
  dteptr->extent      = high - low;
  
  return (errno);
}
