/*
 *  $Id: type_vec.c,v 1.1 1994/04/18 15:47:07 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

#ifndef MPIR_TRUE
#define MPIR_TRUE  1
#define MPIR_FALSE 0
#endif


/*@
    MPI_Type_vector - Create a vector (strided) datatype
@*/
int MPI_Type_vector( count, blocklen, stride, old_type, newtype )
int          count;
int 	     blocklen;
int 	     stride;
MPI_Datatype old_type;
MPI_Datatype *newtype;
{
  MPIR_DATATYPE  *dteptr;
  int errno = MPI_SUCCESS;

  /* Check for bad arguments */
  if ( ( (old_type == MPI_DATATYPE_NULL) && (errno = MPI_ERR_TYPE) )  ||
	   ( (old_type == (*newtype))        && (errno = MPI_ERR_TYPE) )  ||
	   ( (count   <= 0)                  && (errno = MPI_ERR_COUNT) ) ||
	   ( (blocklen <= 0)                 && (errno = MPI_ERR_ARG) )   )
	return MPIR_ERROR( MPI_COMM_NULL, errno,
					  "Error in MPI_TYPE_VECTOR" );
	
  /* Handle the case where blocklen & stride make a contiguous type */
  if ( (blocklen == stride) ||
	   (count    == 1) )
	return MPI_Type_contiguous ( count * blocklen, old_type, newtype );

  /* Reduce this to the hvector case */
  return MPI_Type_hvector ( count, blocklen, stride * old_type->extent,
						    old_type, newtype );
}
