/*
 *  $Id: cart_coords.c,v 1.8 1994/04/12 20:12:58 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Cart_coords - Determines process coords in cartesian topology given
                  rank in group

Input Parameters:
. comm - communicator with cartesian structure (handle) 
. rank - rank of a process within group of comm (integer) 
. maxdims - length of vector  coord in the calling program (integer) 
Output Parameter:
. coords - integer array (of size  ndims ) containing the cartesian coordinates of specified process (integer) 

@*/
int MPI_Cart_coords ( comm, rank, maxdims, coords )
MPI_Comm  comm;
int       rank;
int       maxdims;
int      *coords;
{
  int i, ndims, *dims, num_ranks;
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if ( ((comm               == MPI_COMM_NULL) && (errno = MPI_ERR_COMM)) ||
       ((comm->topology.type != MPI_CART)   && (errno = MPI_ERR_TOPOLOGY))  ||
       ((rank                <  0)          && (errno = MPI_ERR_RANK))      ||
       ((rank >= (comm->topology.tag.cart.nnodes))&&(errno = MPI_ERR_RANK)) ||
       ((maxdims             <  1)          && (errno = MPI_ERR_DIMS))      ||
       ((coords              == (int *)0)   && (errno = MPI_ERR_BUFFER))     )
    return MPIR_ERROR( comm, errno, "Error in MPI_CART_COORDS" );

  /* Calculate coords */
  ndims     = comm->topology.tag.cart.ndims;
  dims      = comm->topology.tag.cart.dims;
  num_ranks = comm->topology.tag.cart.nnodes;
  for ( i=0; (i < ndims) && (i < maxdims); i++ ) {
    num_ranks = num_ranks / dims[i];
	coords[i]  = rank / num_ranks;
    rank   = rank % num_ranks;
  }

  return (errno);
}
