/*
 *  $Id: cart_get.c,v 1.3 1994/04/12 20:12:58 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Cart_get - Retrieve cart topology information associated with a 
               communicator

Input Parameters:
. comm - communicator with cartesian structure (handle) 
. maxdims - length of vectors  dims, periods , and  coords in the calling program (integer) 

Output Parameters:
. dims - number of processes for each cartesian dimension (array of integer) 
. periods - periodicity (true /false ) for each cartesian dimension (array of logical) 
. coords - coordinates of calling process in cartesian structure (array of integer) 

@*/
int MPI_Cart_get ( comm, maxdims, dims, periods, coords )
MPI_Comm comm;
int maxdims;
int *dims, *periods, *coords;
{
  int i, num;
  int *array;
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if (((comm                == MPI_COMM_NULL) && (errno = MPI_ERR_COMM)) ||
      ((comm->topology.type != MPI_CART)      && (errno = MPI_ERR_TOPOLOGY))   )
    return MPIR_ERROR( comm, errno, "Error in MPI_CART_GET" );

  /* Get dims */
  num = comm->topology.tag.cart.ndims;
  array = comm->topology.tag.cart.dims;
  if ( dims != (int *)0 )
    for ( i=0; (i<maxdims) && (i<num); i++ )
      (*dims++) = (*array++);

  /* Get periods */
  num = comm->topology.tag.cart.ndims;
  array = comm->topology.tag.cart.periods;
  if ( periods != (int *)0 )
    for ( i=0; (i<maxdims) && (i<num); i++ )
      (*periods++) = (*array++);

  /* Get coords */
  num = comm->topology.tag.cart.ndims;
  array = comm->topology.tag.cart.position;
  if ( coords != (int *)0 )
    for ( i=0; (i<maxdims) && (i<num); i++ )
      (*coords++) = (*array++);

  return (errno);
}
