/*
 *  $Id: cart_rank.c,v 1.10 1994/04/13 13:27:15 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Cart_rank - determines process rank in communicator given cartesian
                location

Input Parameters:
. comm - communicator with cartesian structure (handle) 
. coords - integer array (of size  ndims ) specifying the cartesian coordinates of a process 
Output Parameter:
. rank - rank of specified process (integer) 

@*/
int MPI_Cart_rank ( comm, coords, rank )
MPI_Comm comm;
int *coords;
int *rank;
{
  int i, ndims, multiplier = 1;
  int errno = MPI_SUCCESS;
  int coord;

  /* Check for valid arguments */
  if ( ((comm                == MPI_COMM_NULL)&&(errno = MPI_ERR_COMM)) ||
       ((rank                == (int *)0)   &&(errno = MPI_ERR_BUFFER))    || 
       ((comm->topology.type != MPI_CART)   &&(errno = MPI_ERR_TOPOLOGY))  ||
       ((comm->topology.tag.cart.ndims <  1)&&(errno = MPI_ERR_DIMS))       )
      return MPIR_ERROR( comm, errno, "Error in MPI_CART_RANK" );

  /* Compute rank */
  ndims = comm->topology.tag.cart.ndims;
  (*rank) = coords[ndims-1];
  if ( !(comm->topology.tag.cart.periods[ndims-1]) ) {
	if ( ((*rank) == MPI_PROC_NULL)                          ||
		 ((*rank) >= comm->topology.tag.cart.dims[ndims-1]) ||
		 ((*rank) <  0) ) {
	  (*rank) = MPI_PROC_NULL;
	  return (errno);
	}
  }
  else {
	if ( (*rank) == MPI_PROC_NULL ) {
	  (*rank) = MPI_PROC_NULL;
	  return (errno);
	}
	if ( ( (*rank) >= comm->topology.tag.cart.dims[ndims-1] ) ||
 		 ( (*rank) <  0 ) ) {
	  (*rank) = (*rank) % comm->topology.tag.cart.dims[ndims-1];
	}
  }
    
  for ( i=ndims-2; i >=0; i-- ) {
	coord = coords[i];
    if ( coord == MPI_PROC_NULL ) {
      (*rank) = MPI_PROC_NULL;
      return (errno);
	}
	if ( !(comm->topology.tag.cart.periods[i]) ) {
	  if ( (coord >= comm->topology.tag.cart.dims[i]) ||
		   (coord <  0) ) {
		(*rank) = MPI_PROC_NULL;
		return (errno);
	  }
    }
	else {
	  if ( (coord >= comm->topology.tag.cart.dims[i]) ||
		   (coord <  0) )
		coord = coord % comm->topology.tag.cart.dims[i];
	}
    multiplier *= comm->topology.tag.cart.dims[i+1];
    (*rank) += multiplier * coord;
  }

  return (errno);
}
