/*
 *  $Id: get_graph.c,v 1.7 1994/03/29 22:25:07 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Graph_get - Retrieve graph topology information associated with a 
                communicator

Input Parameters:
. comm - communicator with graph structure (handle) 
. maxindex - length of vector index in the calling program  (integer) 
. maxedges - length of vector edges in the calling program  (integer) 
Output Parameter:
. index - array of integers containing the graph structure (for details see the definition of MPI_GRAPH_CREATE ) 
. edges - array of integers containing the graph structure 

@*/
int MPI_Graph_get ( comm, maxindex, maxedges, index, edges )
MPI_Comm comm;
int maxindex, maxedges;
int *index, *edges;
{
  int i, num;
  int *array;
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if (((comm                == (MPI_Comm)0) && (errno = MPI_ERR_COMM_NULL)) ||
      ((comm->topology.type != MPI_GRAPH)   && (errno = MPI_ERR_TOPOLOGY))   )
    return MPIR_ERROR( comm, errno, "Error in MPI_GRAPH_GET" );

  /* Get index */
  num = comm->topology.tag.graph.nnodes;
  array = comm->topology.tag.graph.index;
  if ( index != (int *)0 )
    for ( i=0; (i<maxindex) && (i<num); i++ )
      (*index++) = (*array++);

  /* Get edges */
  num = comm->topology.tag.graph.nedges;
  array = comm->topology.tag.graph.edges;
  if ( edges != (int *)0 )
    for ( i=0; (i<maxedges) && (i<num); i++ )
      (*edges++) = (*edges++);

  return (errno);
}
