/*
 *  $Id: graph_nbr_cnt.c,v 1.8 1994/04/12 20:12:58 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Graph_neighbors_count - returns the number of neighbors of a node
                            associated with a graph topology

Input Parameters:
. comm - communicator with graph topology (handle) 
. rank - rank of process in group of comm (integer) 

Output Parameter:
. nneighbors - number of neighbors of specified process (integer) 

@*/
int MPI_Graph_neighbors_count ( comm, rank, nneighbors )
MPI_Comm  comm;
int       rank;
int      *nneighbors;
{
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if (((comm                == MPI_COMM_NULL) && (errno = MPI_ERR_COMM)) ||
      ((comm->topology.type != MPI_GRAPH)     && (errno = MPI_ERR_TOPOLOGY))  ||
      ((rank                <  0)             && (errno = MPI_ERR_RANK))      ||
      ((rank >= (comm->topology.tag.graph.nnodes))&&(errno = MPI_ERR_RANK)) ||
      ((nneighbors          == (int *)0)      && (errno = MPI_ERR_ARG))       )
    return MPIR_ERROR( comm, errno, "Error in MPI_GRAPH_NEIGHBORS_COUNT" );

  /* Get nneighbors */
  if ( rank == 0 ) 
    (*nneighbors) = comm->topology.tag.graph.index[rank];
  else
    (*nneighbors) = comm->topology.tag.graph.index[rank] -
                    comm->topology.tag.graph.index[rank-1];
  
  return (errno);
}
